@@ -16,19 +16,19 @@ Promise::XS - Fast promises in Perl
     my $deferred = Promise::XS::deferred();
 
     # Do one of these once you have the result of your operation:
-    $deferred->resolve( 'foo', 'bar' );
-    $deferred->reject( 'oh', 'no!' );
+    $deferred->resolve( 'foo' );
+    $deferred->reject( 'oh no!' );
 
     # Give this to your caller:
     my $promise = $deferred->promise();
 
 The following aggregator functions are exposed:
 
-    # Resolves with a list of arrayrefs, one per promise.
+    # Resolves with an arrayref, one item per promise.
     # Rejects with the results from the first rejected promise.
     my $all_p = Promise::XS::all( $promise1, $promise2, .. );
 
-    # Resolves/rejects with the results from the first
+    # Resolves/rejects with the result from the first
     # resolved or rejected promise.
     my $race_p = Promise::XS::race( $promise3, $promise4, .. );
 
@@ -37,9 +37,9 @@ as C<collect()>.
 
 The following also exist:
 
-    my $pre_resolved_promise = Promise::XS::resolved('already', 'done');
+    my $pre_resolved_promise = Promise::XS::resolved('already done');
 
-    my $pre_rejected_promise = Promise::XS::rejected('it’s', 'bad');
+    my $pre_rejected_promise = Promise::XS::rejected('badbad');
 
 All of C<Promise::XS>’s static functions may be exported at load time,
 e.g., C<use Promise::XS qw(deferred)>.
@@ -48,25 +48,43 @@ e.g., C<use Promise::XS qw(deferred)>.
 
 This module exposes a Promise interface with its major parts
 implemented in XS for speed. It is a fork and refactor of
-L<AnyEvent::XSPromises>. That module’s interface, a “bare-bones”
-subset of that from L<Promises>, is retained.
+L<AnyEvent::XSPromises>, with some significant interface changes.
 
 =head1 STATUS
 
-Breaking changes in this interface are unlikely; however, the implementation
-is relatively untested since the fork. Your mileage may vary.
+This module should be fairly stable but is still relatively untested since
+the fork from L<AnyEvent::XSPromises>. Significant breaking changes happened
+from version 0.07 to 0.08. Such changes aren’t expected to be needed again,
+but that’s not guaranteed. Caveat emptor.
 
-=head1 DIFFERENCES FROM ECMASCRIPT PROMISES
+=head1 PROMISE INTERFACE
 
-This library is built for compatibility with pre-existing Perl promise
-libraries. It thus exhibits some salient differences from how
-ECMAScript promises work:
+The core functionality derives from the
+L<Promises/A+ specification|https://promisesaplus.com/> standard. (See
+L</EVENT LOOPS> below for one important difference.) Its C<finally()>
+implementation L<derives from ECMAScript promises|https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise/finally>.
+
+=head2 Promise callbacks: list vs. scalar context
+
+Most Perl promise libraries allow promises to resolve (or reject) with a list.
+The problem with this pattern is: what do we do if a plural return includes
+a promise? Neither Promises/A+ nor ECMAScript’s promise standard offers
+guidance on how to handle this scenario, and there’s no “obvious” solution
+otherwise. We could simply ignore the “extra” inputs, but what if one of those
+“extras” is itself a promise? What if there’s only one promise, but it’s not
+the first item returned?
+
+All of these scenarios allow for subtle bugs to arise. To avoid that,
+this library executes all callbacks in scalar context. Besides avoiding the
+“problem” states described above, this also matches both Promises/A+ and
+ECMAScript standards. The divergence from preexisting Perl promise libraries
+like L<Mojo::Promise>, L<Promises>, and L<AnyEvent::XSPromises>, is
+regrettable but seems a “lesser evil” overall.
+
+=head2 Additional notes
 
 =over
 
-=item * Promise resolutions and rejections consist of I<lists>, not
-single values.
-
 =item * Neither the C<resolve()> method of deferred objects
 nor the C<resolved()> convenience function define behavior when given
 a promise object.
@@ -81,31 +99,14 @@ call “array references”). So whereas in ECMAScript you do:
 
     Promise::XS::all( $promise1, $promise2 );
 
+=item * Currently C<finally()> does not recognize returned promises.
+Hopefully that will change in the future to match ECMAScript’s standard.
+
 =back
 
 See L<Promise::ES6> for an interface that imitates ECMAScript promises
 more closely.
 
-=head1 DIFFERENCES FROM L<Promises> ET AL.
-
-This module implements ECMAScript’s C<finally()> interface, which differs
-from that in some other Perl promise implementations.
-
-Given the following …
-
-    my $new = $p->finally( $callback )
-
-=over
-
-=item * C<$callback> is given no arguments and is called in void context.
-
-=item * If C<$callback> returns, C<$new> has the same status as C<$p>.
-
-=item * If C<$callback> throws, C<$new> is rejected with C<$callback>’s
-exception.
-
-=back
-
 =head1 EVENT LOOPS
 
 By default this library uses no event loop. This is a perfectly usable
@@ -167,6 +168,9 @@ as should C<resolved()> and C<rejected()>.
 
 =over
 
+=item * C<finally()> ignores rejected promises given as returns rather than
+rejecting the promise as should happen.
+
 =item * Interpreter-based threads may or may not work.
 
 =item * This module interacts badly with Perl’s fork() implementation on
@@ -237,10 +241,10 @@ sub all {
 
         $p->then(
             sub {
-                $values[$i]= \@_;
+                $values[$i]= $_[0];
                 if ((--$remaining) == 0) {
                     $pending= 0;
-                    $then_what->resolve(@values);
+                    $then_what->resolve(\@values);
                 }
             },
             $reject_now,
